import 'package:collection/collection.dart';
import 'package:localization_builder/src/definitions/template_value.dart';
import 'package:meta/meta.dart';

import 'condition.dart';
import 'label.dart';

/// Case represents a specific case for a label that respect a [condition], with
/// a set of associated [translations].
class Case {
  final Condition condition;
  final List<Translation> translations;
  final List<TemplatedValue> templatedValues;
  Case({
    @required this.condition,
    @required this.translations,
  })  : assert(assertTranslationsValid(translations)),
        templatedValues =
            translations.isEmpty ? [] : translations.first.templatedValues;

  /// Verifies that all translation havez the same templated values (if so).
  static bool assertTranslationsValid(List<Translation> translations) {
    if (translations.length > 1) {
      final templatedValues = translations.first.templatedValues;
      for (var i = 1; i < translations.length; i++) {
        final current = translations[i];
        if (!const SetEquality()
            .equals(templatedValues.toSet(), current.templatedValues.toSet())) {
          return false;
        }
      }
    }

    return true;
  }
}
