import 'package:equatable/equatable.dart';
import 'package:recase/recase.dart' as recase;

import 'category.dart';

abstract class Condition extends Equatable {
  const Condition();
  factory Condition.parse(String value) {
    if (value == null) return const DefaultCondition();
    value = value.trim();
    if (value.isEmpty) return const DefaultCondition();
    final splits = value.split('.');
    assert(splits.length == 2,
        'Category condition should be composed of two segments `<category>.<value>`');
    return CategoryCondition(Category(splits[0]), splits[1]);
  }
}

class DefaultCondition extends Condition {
  const DefaultCondition();

  @override
  List<Object> get props => const <Object>[];
}

class CategoryCondition extends Condition {
  final Category category;
  final String value;
  CategoryCondition(this.category, String value)
      : value = recase.ReCase(value).camelCase;

  @override
  List<Object> get props => [category, value];
}
