import 'package:meta/meta.dart';
import 'package:recase/recase.dart' as recase;

import 'case.dart';
import 'category.dart';
import 'condition.dart';
import 'label.dart';

/// A section is a set of labels, grouped together to to make them
/// easier to find.
class Section {
  Section({
    @required this.path,
    @required String key,
    List<Label> labels,
    List<Section> children,
  })  : key = key ?? 'labels',
        labels = labels ?? <Label>[],
        children = children ?? <Section>[];
  final List<String> path;
  final String key;
  final List<Label> labels;
  final List<Section> children;
  String get normalizedKey => recase.ReCase(key).camelCase;
  String get normalizedName =>
      path.map((x) => recase.ReCase(x).pascalCase).join('_');

  List<Category> get categories {
    final result = <Category>[];

    for (var label in allLabels) {
      final category = label.category;
      if (category != null) {
        final existing = result.firstWhere(
          (x) => x.name == category.name,
          orElse: () {
            final newCategory = Category(category.name);
            result.add(newCategory);
            return newCategory;
          },
        );

        existing.values.addAll(category.values);
      }
    }

    return result;
  }

  List<Label> get allLabels {
    final result = <Label>[];
    result.addAll(labels);
    result.addAll(children.expand((x) => x.allLabels));
    return result;
  }

  void insert(String path, String condition, List<Translation> translations) {
    _insert(path.split('.'), condition, translations);
  }

  void _insert(
    List<String> splits,
    String conditionValue,
    List<Translation> translations,
  ) {
    if (splits.isNotEmpty) {
      final key = splits[0].trim();
      if (splits.length == 1) {
        final existing = labels.firstWhere(
          (x) => x.key == key,
          orElse: () => null,
        );
        final condition = Condition.parse(conditionValue);
        final newCase = Case(
          condition: condition,
          translations: translations,
        );
        if (existing != null) {
          existing.addCase(newCase);
        } else {
          labels.add(
            Label(
              key: key,
              cases: [newCase],
            ),
          );
        }
        return;
      } else {
        final section = children.firstWhere((x) => x.key == key, orElse: () {
          final newSection = Section(
            path: <String>[
              ...path,
              key,
            ],
            key: key,
          );
          children.add(newSection);
          return newSection;
        });
        section._insert(
          splits.skip(1).toList(),
          conditionValue,
          translations,
        );
      }
    }
  }
}
